/**************************************************************

   Program Title:  IMU8420 Data Logger
   
   Program Name:  IMU8420_SSD1306.c	Version: 1.04c
   
   Author:  SS	  					    Date:  Sept 26, 2014
   
   Compiled with AVR Studio 6.1.
   
   This file contains routines that implement a set of basic utility 
   functions for the IMU8420 Data Logger.
   
   Copyright 2004-2014, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2014, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_Utilities.c,v $
 * Revision 1.01  2013/02/22  SS
 *    Configure logging from uSD file, GPS support
 * Revision 1.00  2013/02/22  SS
 *    2-wire interface, 8MHz clock, 19,200baud
 * Revision 0.86  2006/02/07  SS
 *    2-wire interface, 8MHz clock, 19,200baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
// Include header files for GCC/IAR
#include "asf.h"
//#include "compiler.h"
#include "preprocessor.h"

#include "math.h"
#include "string.h"
#include "ctype.h"
#include "stdarg.h"
#include "stdint.h"
#include "stdbool.h"
#include "stdio.h"
#include "stdlib.h"
#include "ctype.h"

#include "IMU8420_Defines.h"
#include "IMU8420_Externals.h"
#include "Quat_EKF.h"
#include "SSD1306.h"

#include "ff.h"
#include "diskio.h"

extern BYTE Buff[];
extern FIL File[];
extern FATFS Fatfs[];

extern volatile struct {
	float latitude;
	char  latchar;
	float longitude;
	char  lonchar;
	float gmtime;
	float altitude;
	float hdop;
	float geodsep;
	int  numsat;
	char fixavailable;
} gps;

/*****************************************************************
Character generator table is stored in progmem as "font", and contains
128 cells in PETSCII layout (i.e. you can generate them from C64 font files).
Included is the canonical 8x8 C64 font, with a couple minor changes to make
box-drawn digits look better. Characters are rotated 90 degrees clockwise
(so we don't have to waste AVR cycles flipping tiles). The pointy part of
an 'A' should point that way -> if you're doing it right.

A character generation utility is here: http://www.min.at/prinz/o/software/pixelfont/

***********************************************************************/
//const unsigned char font[1024] PROGMEM =  // version for AVR's
const unsigned char font[1024]  =           // version for AVR32
{
	0x00, 0x3E, 0x7F, 0x41, 0x4D, 0x4F, 0x2E, 0x00,	// Char 000 (.)
	0x00, 0x20, 0x74, 0x54, 0x54, 0x7C, 0x78, 0x00,	// Char 001 (.)
	0x00, 0x7E, 0x7E, 0x48, 0x48, 0x78, 0x30, 0x00,	// Char 002 (.)
	0x00, 0x38, 0x7C, 0x44, 0x44, 0x44, 0x00, 0x00,	// Char 003 (.)
	0x00, 0x30, 0x78, 0x48, 0x48, 0x7E, 0x7E, 0x00,	// Char 004 (.)
	0x00, 0x38, 0x7C, 0x54, 0x54, 0x5C, 0x18, 0x00,	// Char 005 (.)
	0x00, 0x00, 0x08, 0x7C, 0x7E, 0x0A, 0x0A, 0x00,	// Char 006 (.)
	0x00, 0x98, 0xBC, 0xA4, 0xA4, 0xFC, 0x7C, 0x00,	// Char 007 (.)
	0x00, 0x7E, 0x7E, 0x08, 0x08, 0x78, 0x70, 0x00,	// Char 008 (.)
	0x00, 0x00, 0x48, 0x7A, 0x7A, 0x40, 0x00, 0x00,	// Char 009 (.)
	0x00, 0x00, 0x80, 0x80, 0x80, 0xFA, 0x7A, 0x00,	// Char 010 (.)
	0x00, 0x7E, 0x7E, 0x10, 0x38, 0x68, 0x40, 0x00,	// Char 011 (.)
	0x00, 0x00, 0x42, 0x7E, 0x7E, 0x40, 0x00, 0x00,	// Char 012 (.)
	0x00, 0x7C, 0x7C, 0x18, 0x38, 0x1C, 0x7C, 0x78,	// Char 013 (.)
	0x00, 0x7C, 0x7C, 0x04, 0x04, 0x7C, 0x78, 0x00,	// Char 014 (.)
	0x00, 0x38, 0x7C, 0x44, 0x44, 0x7C, 0x38, 0x00,	// Char 015 (.)
	0x00, 0xFC, 0xFC, 0x24, 0x24, 0x3C, 0x18, 0x00,	// Char 016 (.)
	0x00, 0x18, 0x3C, 0x24, 0x24, 0xFC, 0xFC, 0x00,	// Char 017 (.)
	0x00, 0x7C, 0x7C, 0x04, 0x04, 0x0C, 0x08, 0x00,	// Char 018 (.)
	0x00, 0x48, 0x5C, 0x54, 0x54, 0x74, 0x24, 0x00,	// Char 019 (.)
	0x00, 0x04, 0x04, 0x3E, 0x7E, 0x44, 0x44, 0x00,	// Char 020 (.)
	0x00, 0x3C, 0x7C, 0x40, 0x40, 0x7C, 0x7C, 0x00,	// Char 021 (.)
	0x00, 0x1C, 0x3C, 0x60, 0x60, 0x3C, 0x1C, 0x00,	// Char 022 (.)
	0x00, 0x1C, 0x7C, 0x70, 0x38, 0x70, 0x7C, 0x1C,	// Char 023 (.)
	0x00, 0x44, 0x6C, 0x38, 0x38, 0x6C, 0x44, 0x00,	// Char 024 (.)
	0x00, 0x9C, 0xBC, 0xA0, 0xE0, 0x7C, 0x3C, 0x00,	// Char 025 (.)
	0x00, 0x44, 0x64, 0x74, 0x5C, 0x4C, 0x44, 0x00,	// Char 026 (.)
	0x00, 0x00, 0x7F, 0x7F, 0x41, 0x41, 0x00, 0x00,	// Char 027 (.)
	0x40, 0x68, 0x7C, 0x5E, 0x49, 0x49, 0x22, 0x00,	// Char 028 (.)
	0x00, 0x00, 0x41, 0x41, 0x7F, 0x7F, 0x00, 0x00,	// Char 029 (.)
	0x00, 0x08, 0x0C, 0xFE, 0xFE, 0x0C, 0x08, 0x00,	// Char 030 (.)
	0x00, 0x18, 0x3C, 0x7E, 0x18, 0x18, 0x18, 0x18,	// Char 031 (.)
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 032 ( )
	0x00, 0x00, 0x00, 0x4F, 0x4F, 0x00, 0x00, 0x00,	// Char 033 (!)
	0x00, 0x07, 0x07, 0x00, 0x00, 0x07, 0x07, 0x00,	// Char 034 (")
	0x14, 0x7F, 0x7F, 0x14, 0x14, 0x7F, 0x7F, 0x14,	// Char 035 (#)
	0x00, 0x24, 0x2E, 0x6B, 0x6B, 0x3A, 0x12, 0x00,	// Char 036 ($)
	0x00, 0x63, 0x33, 0x18, 0x0C, 0x66, 0x63, 0x00,	// Char 037 (%)
	0x00, 0x32, 0x7F, 0x4D, 0x4D, 0x77, 0x72, 0x50,	// Char 038 (&)
	0x00, 0x00, 0x00, 0x04, 0x06, 0x03, 0x01, 0x00,	// Char 039 (')
	0x00, 0x00, 0x1C, 0x3E, 0x63, 0x41, 0x00, 0x00,	// Char 040 (()
	0x00, 0x00, 0x41, 0x63, 0x3E, 0x1C, 0x00, 0x00,	// Char 041 ())
	0x08, 0x2A, 0x3E, 0x1C, 0x1C, 0x3E, 0x2A, 0x08,	// Char 042 (*)
	0x00, 0x08, 0x08, 0x3E, 0x3E, 0x08, 0x08, 0x00,	// Char 043 (+)
	0x00, 0x00, 0x80, 0xE0, 0x60, 0x00, 0x00, 0x00,	// Char 044 (,)
	0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x00,	// Char 045 (-)
	0x00, 0x00, 0x00, 0x60, 0x60, 0x00, 0x00, 0x00,	// Char 046 (.)
	0x00, 0x40, 0x60, 0x30, 0x18, 0x0C, 0x06, 0x02,	// Char 047 (/)
	0x00, 0x3E, 0x7F, 0x49, 0x45, 0x7F, 0x3E, 0x00,	// Char 048 (0)
	0x00, 0x40, 0x44, 0x7F, 0x7F, 0x40, 0x40, 0x00,	// Char 049 (1)
	0x00, 0x62, 0x73, 0x51, 0x49, 0x4F, 0x46, 0x00,	// Char 050 (2)
	0x00, 0x22, 0x63, 0x49, 0x49, 0x7F, 0x36, 0x00,	// Char 051 (3)
	0x00, 0x18, 0x18, 0x14, 0x16, 0x7F, 0x7F, 0x10,	// Char 052 (4)
	0x00, 0x27, 0x67, 0x45, 0x45, 0x7D, 0x39, 0x00,	// Char 053 (5)
	0x00, 0x3E, 0x7F, 0x49, 0x49, 0x7B, 0x32, 0x00,	// Char 054 (6)
	0x00, 0x03, 0x03, 0x79, 0x7D, 0x07, 0x03, 0x00,	// Char 055 (7)
	0x00, 0x36, 0x7F, 0x49, 0x49, 0x7F, 0x36, 0x00,	// Char 056 (8)
	0x00, 0x26, 0x6F, 0x49, 0x49, 0x7F, 0x3E, 0x00,	// Char 057 (9)
	0x00, 0x00, 0x00, 0x24, 0x24, 0x00, 0x00, 0x00,	// Char 058 (:)
	0x00, 0x00, 0x80, 0xE4, 0x64, 0x00, 0x00, 0x00,	// Char 059 (;)
	0x00, 0x08, 0x1C, 0x36, 0x63, 0x41, 0x41, 0x00,	// Char 060 (<)
	0x00, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x00,	// Char 061 (=)
	0x00, 0x41, 0x41, 0x63, 0x36, 0x1C, 0x08, 0x00,	// Char 062 (>)
	0x00, 0x02, 0x03, 0x51, 0x59, 0x0F, 0x06, 0x00,	// Char 063 (?)
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,	// Char 064 (@)
	0x00, 0x7C, 0x7E, 0x0B, 0x0B, 0x7E, 0x7C, 0x00,	// Char 065 (A)
	0x00, 0x7F, 0x7F, 0x49, 0x49, 0x7F, 0x36, 0x00,	// Char 066 (B)
	0x00, 0x3E, 0x7F, 0x41, 0x41, 0x63, 0x22, 0x00,	// Char 067 (C)
	0x00, 0x7F, 0x7F, 0x41, 0x63, 0x3E, 0x1C, 0x00,	// Char 068 (D)
	0x00, 0x7F, 0x7F, 0x49, 0x49, 0x41, 0x41, 0x00,	// Char 069 (E)
	0x00, 0x7F, 0x7F, 0x09, 0x09, 0x01, 0x01, 0x00,	// Char 070 (F)
	0x00, 0x3E, 0x7F, 0x41, 0x49, 0x7B, 0x3A, 0x00,	// Char 071 (G)
	0x00, 0x7F, 0x7F, 0x08, 0x08, 0x7F, 0x7F, 0x00,	// Char 072 (H)
	0x00, 0x00, 0x41, 0x7F, 0x7F, 0x41, 0x00, 0x00,	// Char 073 (I)
	0x00, 0x20, 0x60, 0x41, 0x7F, 0x3F, 0x01, 0x00,	// Char 074 (J)
	0x00, 0x7F, 0x7F, 0x1C, 0x36, 0x63, 0x41, 0x00,	// Char 075 (K)
	0x00, 0x7F, 0x7F, 0x40, 0x40, 0x40, 0x40, 0x00,	// Char 076 (L)
	0x00, 0x7F, 0x7F, 0x06, 0x0C, 0x06, 0x7F, 0x7F,	// Char 077 (M)
	0x00, 0x7F, 0x7F, 0x0E, 0x1C, 0x7F, 0x7F, 0x00,	// Char 078 (N)
	0x00, 0x3E, 0x7F, 0x41, 0x41, 0x7F, 0x3E, 0x00,	// Char 079 (O)
	0x00, 0x7F, 0x7F, 0x09, 0x09, 0x0F, 0x06, 0x00,	// Char 080 (P)
	0x00, 0x1E, 0x3F, 0x21, 0x61, 0x7F, 0x5E, 0x00,	// Char 081 (Q)
	0x00, 0x7F, 0x7F, 0x19, 0x39, 0x6F, 0x46, 0x00,	// Char 082 (R)
	0x00, 0x26, 0x6F, 0x49, 0x49, 0x7B, 0x32, 0x00,	// Char 083 (S)
	0x00, 0x01, 0x01, 0x7F, 0x7F, 0x01, 0x01, 0x00,	// Char 084 (T)
	0x00, 0x3F, 0x7F, 0x40, 0x40, 0x7F, 0x3F, 0x00,	// Char 085 (U)
	0x00, 0x1F, 0x3F, 0x60, 0x60, 0x3F, 0x1F, 0x00,	// Char 086 (V)
	0x00, 0x7F, 0x7F, 0x30, 0x18, 0x30, 0x7F, 0x7F,	// Char 087 (W)
	0x00, 0x63, 0x77, 0x1C, 0x1C, 0x77, 0x63, 0x00,	// Char 088 (X)
	0x00, 0x07, 0x0F, 0x78, 0x78, 0x0F, 0x07, 0x00,	// Char 089 (Y)
	0x00, 0x61, 0x71, 0x59, 0x4D, 0x47, 0x43, 0x00,	// Char 090 (Z)
	0x18, 0x18, 0x18, 0xFF, 0xFF, 0x18, 0x18, 0x18,	// Char 091 ([)
	0x33, 0x33, 0xCC, 0xCC, 0x00, 0x00, 0x00, 0x00,	// Char 092 (\)
	0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00,	// Char 093 (])
	0x00, 0x00, 0x00, 0x0F, 0x0F, 0x00, 0x00, 0x00,	// Char 094 (^)
	0x00, 0x00, 0x00, 0xF0, 0xF0, 0x00, 0x00, 0x00,	// Char 095 (_)
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,	// Char 096 (`)
	0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,	// Char 097 (a)
	0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0,	// Char 098 (b)
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,	// Char 099 (c)
	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,	// Char 100 (d)
	0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 101 (e)
	0x33, 0x33, 0xCC, 0xCC, 0x33, 0x33, 0xCC, 0xCC,	// Char 102 (f)
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF,	// Char 103 (g)
	0x30, 0x30, 0xC0, 0xC0, 0x30, 0x30, 0xC0, 0xC0,	// Char 104 (h)
	0x33, 0x99, 0xCC, 0x66, 0x33, 0x99, 0xCC, 0x66,	// Char 105 (i)
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF,	// Char 106 (j)
	0x00, 0x00, 0x00, 0xFF, 0xFF, 0x18, 0x18, 0x18,	// Char 107 (k)
	0x00, 0x00, 0x00, 0x00, 0xF0, 0xF0, 0xF0, 0xF0,	// Char 108 (l)
	0x00, 0x00, 0x00, 0x1F, 0x1F, 0x18, 0x18, 0x18,	// Char 109 (m)
	0x18, 0x18, 0x18, 0xF8, 0xF8, 0x00, 0x00, 0x00,	// Char 110 (n)
	0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0,	// Char 111 (o)
	0x00, 0x00, 0x00, 0xF8, 0xF8, 0x18, 0x18, 0x18,	// Char 112 (p)
	0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x18,	// Char 113 (q)
	0x18, 0x18, 0x18, 0xF8, 0xF8, 0x18, 0x18, 0x18,	// Char 114 (r)
	0x18, 0x18, 0x18, 0xFF, 0xFF, 0x00, 0x00, 0x00,	// Char 115 (s)
	0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 116 (t)
	0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00,	// Char 117 (u)
	0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF,	// Char 118 (v)
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,	// Char 119 (w)
	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,	// Char 120 (x)
	0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0,	// Char 121 (y)
	0x00, 0x78, 0x78, 0x30, 0x18, 0x0C, 0x06, 0x03,	// Char 122 (z)
	0xF0, 0xF0, 0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00,	// Char 123 ({)
	0x00, 0x00, 0x00, 0x00, 0x0F, 0x0F, 0x0F, 0x0F,	// Char 124 (|)
	0x18, 0x18, 0x18, 0x1F, 0x1F, 0x00, 0x00, 0x00,	// Char 125 (})
	0x0F, 0x0F, 0x0F, 0x0F, 0x00, 0x00, 0x00, 0x00,	// Char 126 (~)
	0x0F, 0x0F, 0x0F, 0x0F, 0xF0, 0xF0, 0xF0, 0xF0
};

/***************************************************************************************
 SSD1306 I2C Control Functions
 
 This ssd1306 set of functions is drawn from two sources:
 some code written by soonc found here ->  http://www.ccsinfo.com/forum/viewtopic.php?t=52836
 Bulk of code written by kmm found here -> https://github.com/kmm/SS1306
 
 Both code bases were combined by SS to create this version.  This is not a buffer based 
 implementation like Adafruit's but is an async write version to keep overhead down.
*******************************************************************************************/

void  ssd1306_command(unsigned char c)
{
	messageBuf[0] = 0x80; // This byte is Command
	messageBuf[1] = c;
	twim_write(BMP085_TWI, messageBuf,2, SSD1306_I2C_ADDRESS, false);
}

void  ssd1306_data(unsigned char c)
{
	messageBuf[0] = 0x40; // This byte is DATA
	messageBuf[1] = c;
	twim_write(BMP085_TWI, messageBuf,2, SSD1306_I2C_ADDRESS, false);
}

// Init according to SSD1306 data sheet and many places on the web
void  ssd1306_Init(void)
{
	// Init sequence for 128x64 OLED module
	ssd1306_command(SSD1306_DISPLAYOFF);                    // 0xAE
	ssd1306_command(SSD1306_SETDISPLAYCLOCKDIV);            // 0xD5
	ssd1306_command(0x80);                 // the suggested ratio 0x80
	ssd1306_command(SSD1306_SETMULTIPLEX);                  // 0xA8
	ssd1306_command(0x3F);
	ssd1306_command(SSD1306_SETDISPLAYOFFSET);              // 0xD3
	ssd1306_command(0x0);                                   // no offset
	ssd1306_command(SSD1306_SETSTARTLINE);// | 0x0);        // line #0
	ssd1306_command(SSD1306_CHARGEPUMP);                    // 0x8D
	ssd1306_command(0x14);  // using internal VCC
	ssd1306_command(SSD1306_MEMORYMODE);                    // 0x20
	ssd1306_command(0x00);          // 0x00 horizontal addressing
	ssd1306_command(SSD1306_SEGREMAP | 0x1); // rotate screen 180
	ssd1306_command(SSD1306_COMSCANDEC); // rotate screen 180
	ssd1306_command(SSD1306_SETCOMPINS);                    // 0xDA
	ssd1306_command(0x12);
	ssd1306_command(SSD1306_SETCONTRAST);                   // 0x81
	ssd1306_command(0x7f);  // was 0xCF
	ssd1306_command(SSD1306_SETPRECHARGE);                  // 0xd9
	ssd1306_command(0x11);  // was F1
	ssd1306_command(SSD1306_SETVCOMDETECT);                 // 0xDB
	ssd1306_command(0x20);  // was 0x40
	ssd1306_command(SSD1306_NORMALDISPLAY);                 // 0xA6
	ssd1306_command(SSD1306_DISPLAYALLON_RESUME);           // 0xA4
	ssd1306_command(SSD1306_DISPLAYON);                     //switch on OLED
}

void ssd1306_set_clock(uint8_t val) 
{
	ssd1306_command(0xD5);
	ssd1306_command(val);
}

void ssd1306_set_addr_mode(uint8_t mode) 
{
	// 0 -> horizontal (write column, increment column pointer, at column overrun reset column pointer and increment page pointer)
	// 1 -> vertical (write column, increment page pointer, at page overrun reset page pointer and increment column pointer)
	// 2 -> page (write column, increment column pointer, reset column pointer at overrun)
	ssd1306_command(0x20);
	ssd1306_command((mode < 2) ? mode : 0);
}

void ssd1306_home(void) 
{
	ssd1306_command(0x21); // column range
	ssd1306_command(0x00); //  set start to 0
	ssd1306_command(0x7F); //  set end to 0x7F
	ssd1306_command(0x22); // row range
	ssd1306_command(0x00); //  set start to 0
	ssd1306_command(0x07); //  set end to 0x07
}

void ssd1306_move_raw(uint8_t row, uint8_t col) 
{
	if(col > 127) { col = 127; }
	if(row > 7) { row = 7; }
	
	ssd1306_command(0x21);     // set column
	ssd1306_command(col);      // start = col
	ssd1306_command(0x7F);     // end = col max
	ssd1306_command(0x22);     // set row
	ssd1306_command(row);      // start = row
	ssd1306_command(0x07);     // end = row max
}

// Transfers the local buffer to the CGRAM in the SSD1306
/*
void TransferBuffer(void)
{
	// set the Column and Page addresses to 0,0
	ssd1306_home(); 
	//setPageAddress();
	
	display_buffer[0] = 0X40; // data not command

	twim_write(BMP085_TWI, display_buffer,1025, SSD1306_I2C_ADDRESS, false);
}
*/

void ssd1306_move(uint8_t row, uint8_t col) 
{
	if(col > 15) { col = 15; }
	if(row > 7) { row = 7; }
	
	ssd1306_move_raw(row, col << 3);
}

void ssd1306_clear(uint8_t value) 
{
	ssd1306_home();
	for(uint16_t i = 1024; i > 0; i--) {
		ssd1306_data(value);
		}
}

void ssd1306_fill(uint8_t row, uint8_t col, uint8_t count, uint8_t max, uint32_t pattern, int8_t pshift) 
{
	uint8_t pstate = 0;
	
	ssd1306_move(row, col);
	for(max; max > 0; max--) {
		if(count > max) {
			if(pshift < 0) {
				ssd1306_data((pattern >> (pstate++ << 3)) & 0xFF);
				}
			else {
				ssd1306_data((pattern >> (pstate++ + pshift)) & 0xFF);
				}
			pstate = (pstate > 3) ? 0 : pstate;
			}
		else {
			ssd1306_data(0x00);
			}
		}
}

void ssd1306_put_tile(uint8_t *tile, uint8_t limit) 
{
	uint16_t i;
	
	for(i=0; i < limit; i++) {
		ssd1306_data(*tile++);
		}
}

void ssd1306_putc_raw(char c) 
{
	uint16_t i;
	
	for( i= c << 3; i < (c << 3) + 8; i++) {
		ssd1306_data(font[i]);
		}
}

void ssd1306_putc(char c) 
{
	// remap from petscii to ascii, shifts drawing characters into the lower 32 ascii cells
	if(c > 'A' && c < 'Z') { }               // upper-case ascii range
	else if(c >= 'a' && c <= 'z') { c -= 96; } // lower-case ascii range
	else if(c > 31 && c < 64) { }            // numbers and symbols
	else if(c < 32) { c += 96; }             // low ascii
	ssd1306_putc_raw(c);
}

// Draw a tile at an arbitrary pixel location (top, left) using an 8 byte tile buffer referenced by *tile.
// Slower than oled_putc(), potentially substantially so; only use for things that need
// finer grained positioning than is possible with tile cells, like sprites.
// Clips right and bottom edges properly; untested and not expected to work with negative positions.
void ssd1306_putxy(uint8_t left_pxl, uint8_t top_pxl, uint8_t *tile) 
{
	uint8_t tbuf[8], obuf[8];
	uint8_t top_cell = top_pxl >> 3;
	uint8_t left_cell = left_pxl >> 3;
	int8_t voff = top_pxl - ((top_cell << 3) - 1);
	int8_t hoff = left_pxl - ((left_cell << 3) - 1);
	uint8_t tcol;
	
	if(voff == 0 && hoff == 0) {
		ssd1306_move(top_cell, left_pxl >> 3);
		ssd1306_put_tile(tile, 8);
		return;
		}
	else {
		for(tcol = 0; tcol < 8; tcol++) { // tile column
			tbuf[tcol] = (tile[tcol]) << ((uint8_t)voff); // shift left (down) by voff
			obuf[tcol] = (tile[tcol]) >> (8 - (uint8_t)voff); //shift right (up) by voff
			}
		
		ssd1306_move_raw(top_cell, left_pxl); // move_raw(row[0:7], column[0:127]) rows and pixels for extra confusion
		ssd1306_put_tile(&tbuf, (left_pxl > (SS1306_OLED_GEOM_W - 8)) ? 8 - hoff : 8);
		if(top_pxl < (SS1306_OLED_GEOM_H - 8)) {
			ssd1306_move_raw((top_cell + 1), left_pxl);
			ssd1306_put_tile(&obuf, (left_pxl > (SS1306_OLED_GEOM_W - 8)) ? 8 - hoff : 8);
			}
		}
}

void ssd1306_puts(char *str) 
{
	while(*str != 0) {
		ssd1306_putc(*str++);
		}
}

// box graphics digit (a single digit, not byte or word; use this to render output of an int->bcd conversion etc)
void ssd1306_bigdigit(uint8_t top, uint8_t left, uint8_t num) 
{
	uint8_t i;
	
//	const uint8_t chartable[] PROGMEM = { 0x10, 0x0E, 0x5D, 0x5D, 0x0D, 0x1D, // zero
	const uint8_t chartable[]  = { 
		0x10, 0x0E, 0x5D, 0x5D, 0x0D, 0x1D, // zero
		0x20, 0x0E, 0x20, 0x5D, 0x20, 0x11, // one
		0x10, 0x0E, 0x10, 0x1D, 0x0D, 0x1D, // two
		0x10, 0x0E, 0x20, 0x13, 0x0D, 0x1D, // three
		0x5F, 0x5F, 0x0D, 0x13, 0x20, 0x5E, // four
		0x10, 0x0E, 0x0D, 0x0E, 0x0D, 0x1D, // five
		0x10, 0x0E, 0x0B, 0x0E, 0x0D, 0x1D, // six
		0x10, 0x0E, 0x20, 0x5B, 0x20, 0x5E, // seven
		0x10, 0x0E, 0x0B, 0x13, 0x0D, 0x1D, // eight
		0x10, 0x0E, 0x0d, 0x13, 0x0D, 0x1D  // nine
	};
	if(num > 9) { return; }
	ssd1306_move(top, left);
	for(i = 0; i < 6; i++) {
		if(i == 2 || i == 4) { ssd1306_move(++top, left); }
		ssd1306_putc(chartable[(num * 6) + i]);
	}
}

void ssd1306_box(uint8_t top, uint8_t left, uint8_t width, uint8_t height) 
{
	uint8_t i;
	
	ssd1306_move(top, left);
	ssd1306_putc(BOX_TL); for(i = 0; i < width - 2; i++) { ssd1306_putc(BOX_HL);} ssd1306_putc(BOX_TR);
	for(i = top+1; i < top+height-1; i++) {
		ssd1306_move(i, left);
		ssd1306_putc(BOX_VL);
		ssd1306_move(i, left + width - 1);
		ssd1306_putc(BOX_VL);
		}
	ssd1306_move(top + height - 1, left);
	ssd1306_putc(BOX_BL); for(i = 0; i < width - 2; i++) { ssd1306_putc(BOX_HL);} ssd1306_putc(BOX_BR);
}

void ssd1306_printfs(unsigned char row, unsigned char col, char *inchar)
{
	ssd1306_move(row, col);
	ssd1306_puts(inchar);
}

int ssd1306_printf(unsigned char row, unsigned char col, const char *fmt, ...)
{
	va_list args;
	int i;
	char printbuffer[40];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 250, fmt, args);
	va_end (args);

	ssd1306_move(row, col);
	ssd1306_puts(printbuffer);

	return i;
}

// TBD - limit number of total digits
void ssd1306_putbigfloat(unsigned char row,unsigned char col,float num,int digits,int decimal)
{
	char outstrg[10];
	unsigned char i,j,k;

	sprintf(outstrg,"%f",num);
	i = 0;
	while(outstrg[i]!='.') {
		ssd1306_bigdigit(row,col+i*2,outstrg[i]-0x30);
		i++;
		}
	k = col+i*2;
	j = 0;
	while((i+1)<strlen(outstrg)) {
		ssd1306_bigdigit(row,col+(i-1)*2+1,outstrg[i]-0x30);
		i++;
		if(++j>decimal) break;
		}
	ssd1306_move(row,k);
	ssd1306_putc(' ');
	ssd1306_move(row+1,k);
	ssd1306_putc(' ');
	ssd1306_move(row+2,k);
	ssd1306_putc('.');
}

// TBD - output select number of digits - ie shift right
void ssd1306_putbigdigit(unsigned char row,unsigned char col,int num,int digits)
{
	char outstrg[7];
	int i;

	sprintf(outstrg,"%5d",num);
	//printf("\n\routstrg=%s\n\r",outstrg);
	for(i=0; i<strlen(outstrg); i++) ssd1306_bigdigit(row,col+i*2,outstrg[i]-0x30);
}

void SSD1306_Test(void)
{
	char inchar,outstrg[20];
	int i;
	float x,y,z;
	
	//memset(display_buffer, 0X55, 1025); // tried other values 	
	//memset(display_buffer, 0X55, 1025); // tried other values

    //ssd1306_Init();
   
	//ssd1306_clear(255);
	//Delaymsec(500);
	//ssd1306_clear(0);
		
	//ssd1306_printf(0,0,"IMU8420 10DOF");	
	//ssd1306_printf(1,0,"abcdefghijklmn");	
	//ssd1306_printf(2,0,"opqrstuvwxyz");
	//ssd1306_printf(3,0,"ABCDEFGHIJKLMN");
	//ssd1306_printf(4,0,"OPQRSTUVWXYZ");

	//ssd1306_bigdigit(3,0,1);
	//ssd1306_bigdigit(3,2,2);
	//ssd1306_bigdigit(3,4,3);

	x = 1.23;
	i = 0;
	while((inchar=getcharnw())!='e') {
		//ssd1306_putbigdigit(3,0,i++,4);
		ssd1306_putbigfloat(3,0,x,8,3);
		x += 0.02;
	}

	ssd1306_box(6, 0, 3, 2);

	x = 1.23;
	y = -2.34;
	z = 8.23;
	if(0) {
		while((inchar=getcharnw())!='e') {
			ssd1306_printf(2,0,"x=%8.2f  ",x);
			ssd1306_printf(3,0,"y=%8.2f  ",y);
			ssd1306_printf(4,0,"z=%8.2f  ",z);
			x += 0.02;
			y -= 0.001;
			z += 0.15;
			delay_ms(50);
			}
		}
}