/*************************************************************************
 *
 *   Program:  PicoM_Defines.h		Version: 1.00
 *
 *   Author:  SS					Date:  Nov 30, 2014
 *
 *   This program contains the PicoM defines used the the various files making up this project.
 *
 *   This project is licensed under the MIT license.  You can modify, enhance, etc anyway you like.
 *
 *   SOC Robotics, Inc is not liable in anyway for the use of this software and assumes no responsibility 
 *   for it's use.  No warranties of any kind are provided.
 *
 *   Copyright 2014, SOC Robotics, Inc.  
 *   There maybe code from other parties in this application  If we don't mention them by name thank you
 *   and if we should please let us know and we will include your name and attribution.
 *
 *************************************************************************/

/**************************************************************************
	Defines
**************************************************************************/

// Target - Select one to enable peripheral support for different Pico sensor mixes
#include "PicoM_Type.h"

#define BAUDRATE9600  207
#define BAUDRATE19200 103
#define BAUDRATE38400  51
#define BAUDRATE115200 16

#define  XON    0x11
#define  XOFF   0x13
#define  ON		'o'
#define	 OFF	'f'
#define  LF	    0x0a
#define  CR	    0x0d
#define  BS     0x08
#define  ESC	0x1b
#define  YES			'y'
#define  NO				'n'
#define  ACTIVESTATE	'a'
#define  INACTIVE       'i'
#define  VERBOSE        'v'
#define  QUIET			'q'

#define  GreenLedON		GPIO_ResetBits(GPIOB,GPIO_Pin_0);
#define  GreenLedOFF	GPIO_SetBits(GPIOB,GPIO_Pin_0);


#define UPDATEMODE     'u'
#define INITIALIZEMODE 'i'
#define RADTODEG   57.29577

// *****************************************
//  Setup TWI stuff
// *****************************************
#define MASTERTWI			'm'
#define SLAVETWI			's'
#define TWI_MASTER           TWIC
#define TWI_MASTER_PORT      PORTC
#define TWI_SLAVE            TWIC
#define TWI_SPEED            100000
#define TWI_SPEED_SLOW       100000
#define TWI_SPEED_FAST       400000
#define TWI_MASTER_ADDR      0x50
#define TWI_SLAVE_ADDR       0x21
#define SMARTLCDTWIADDRESS   0x26

// Bosch BMP180 Sensor TWI Address - replaced by BMP180 on IMU8420
#define BMP180TWIADDRESS	0x77
#define	BMPTEMP				't'
#define BMPPRESSURE			'p'
#define BMPTEMPIDLE			'i'
#define BMPTEMPREAD			'r'
#define BMPPRESSUREREAD		'r'
#define BMPPRESSUREIDLE		'i'
#define BMPSKIPVALUE		1
#define BMP180TEMPOFFSET   -7.00
#define BMPTEMPDELAY		9
#define BMPPRESSUREDELAY    28

// PCA9540 Parameters
#define PCA9540_SLAVE_ADDRESS  0x70
#define PCA9540_NOCHANNEL      0x00
#define PCA9540_CHANNEL_0      0x04
#define PCA9540_CHANNEL_1      0x05

// MPU9150 Parameters
#define MPU9150TEMPOFFSET     -7.00
#define USBOUT				  'u'
#define NOUSBOUT			  'n'

// FXOS8700CQ Parameters

// FXOS8700CQ I2C address
//#define FXOS8700CQ_SLAVE_ADDR 0x1E // with pins SA0=0, SA1=0
//#define FXOS8700CQ_SLAVE_ADDR 0x1D // with pins SA0=1, SA1=0
//#define FXOS8700CQ_SLAVE_ADDR 0x1C // with pins SA0=0, SA1=1
#define FXOS8700CQ_SLAVE_ADDR 0x1F // with pins SA0=1, SA1=1

#define FXOS8700CQ_MCP_SLAVE_ADDR  0x1C   // MCP - Meta Carpal Phalangeal
#define FXOS8700CQ_PIP_SLAVE_ADDR  0x1D   // PIP - Proximal Inter Phalangeal
#define FXOS8700CQ_DIP_SLAVE_ADDR  0x1F   // DIP - Distal Inter Phalangeal

#define MAGSCALE27  27.3
#define MAGSCALE10  10.0
#define ACCELSCALE  4.88e-4

// Local Vancouver inclination and declination -
#define LOCALINCLINATION 70.367
#define LOCALDECLINATION 16.777
// Local field strength
#define LOCALHNT  18.379
#define LOCALVNT  51.579
#define LOCALFIELDSTRENGTH 54.699
#define LOCALFIELDX  17.596
#define LOCALFIELDY  5.305

// FXOS8700CQ internal register addresses
#define FXOS8700CQ_STATUS 0x00
#define FXOS8700CQ_WHOAMI 0x0D
#define FXOS8700CQ_XYZ_DATA_CFG 0x0E
#define FXOS8700CQ_CTRL_REG1 0x2A
#define FXOS8700CQ_M_CTRL_REG1 0x5B
#define FXOS8700CQ_M_CTRL_REG2 0x5C
#define FXOS8700CQ_WHOAMI_VAL 0xC7
#define FXOS8700CQ_MDR_STATUS 0x32
#define FXOS8700CQ_MXOUT_DATA 0x33
#define FXOS8700CQ_AXOUT_DATA 0x39

// Number of bytes to be read from the FXOS8700CQ
#define FXOS8700CQ_READ_LEN 12   // status plus 6 channels = 13 bytes
#define I2C_ERROR  1
#define I2C_OK  0
#define I2C_FAIL 1
#define STATUS_OK 1

// Defining number of bytes in buffer
#define NUM_BYTES        8

#define PRECISION 4
